﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.Web.UI;
using System.Globalization;

namespace Terralever.Facebook.UI.FbmlControls
{
    /// <summary>
    /// Displays content inside the tag only if the logged-in user is permitted by the specified privacy setting of the specified user.
    /// </summary>
    /// <remarks>
    /// <para>The Facebook documentation for the rendered control can be found 
    /// <a href="http://wiki.developers.facebook.com/index.php/Fb:if-is-friends-with-viewer" target="_blank">here</a>.</para>
    /// </remarks>
    [ToolboxData("<{0}:IfIsFriendsWithViewer runat=\"server\"></{0}:IfIsFriendsWithViewer>")]
    [DefaultProperty("UserID")]
    public class IfIsFriendsWithViewer : LogicTestControl
    {
        /// <summary>
        /// Creates a new <see>IfIsFriendsWithViewer</see>.
        /// </summary>
        public IfIsFriendsWithViewer()
        {
            IncludeSelf = true;
        }

        /// <summary>
        /// Gets or sets the user ID to check.  If this value is not set, it will default to the profile owner.
        /// </summary>
        [Browsable(true)]
        [Category(FbmlConstants.FBML_OPT_ATTR)]
        [Description("Specifies the photo ID to check.")]
        [DefaultValue(0)]
        public long UserID
        {
            get;
            set;
        }

        /// <summary>
        /// Optionally gets or sets the ID of another user if the current user should be able to see the photo through another user.
        /// </summary>
        [Browsable(true)]
        [Category(FbmlConstants.FBML_OPT_ATTR)]
        [Description("Specifies whether the control should treat the owner as friends with himself or herself.")]
        [DefaultValue(true)]
        public bool IncludeSelf
        {
            get;
            set;
        }

        /// <inheritdoc />
        protected override void AddAttributesToRender(HtmlTextWriter writer)
        {
            base.AddAttributesToRender(writer);

            if (UserID != 0)
                writer.AddAttribute("uid", UserID.ToString(CultureInfo.InvariantCulture));
            if (!IncludeSelf)
                writer.AddAttribute("includeself", FbmlConstants.FALSE);
        }

        /// <inheritdoc />
        protected internal override string ElementName
        {
            get { return FbmlConstants.FB_IF_IS_FRIENDS_WITH_VIEWER; }
        }
    }
}
