﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using Terralever.Facebook.UI.Design;
using System.Globalization;
using System.Web.UI;

namespace Terralever.Facebook.UI.FbmlControls
{
    /// <summary>
    /// Displays content inside the tag only if the logged-in user is permitted by the specified privacy setting of the specified user.
    /// </summary>
    /// <remarks>
    /// <para>The Facebook documentation for the rendered control can be found 
    /// <a href="http://wiki.developers.facebook.com/index.php/Fb:if-is-group-member" target="_blank">here</a>.</para>
    /// </remarks>
    [ToolboxData("<{0}:IfIsGroupMember runat=\"server\"></{0}:IfIsGroupMember>")]
    [DefaultProperty("UserID")]
    public class IfIsGroupMember : LogicTestControl
    {
        /// <summary>
        /// Gets or sets the Group ID to check.  This property is required.
        /// </summary>
        [FbmlRequired(IsRequired = true)]
        [Browsable(true)]
        [Category(FbmlConstants.FBML_REQD_ATTR)]
        [Description("Specifies the Group ID to check.")]
        [DefaultValue(0)]
        public long GroupID
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the User ID to check.  If this property is not specified, it defaults to the currently logged-in user.
        /// </summary>
        [Browsable(true)]
        [Category(FbmlConstants.FBML_OPT_ATTR)]
        [Description("Specifies the User ID to check.")]
        [DefaultValue(0)]
        public long UserID
        {
            get;
            set;
        }

        private GroupMembershipRole m_role = GroupMembershipRole.Member;
        /// <summary>
        /// Gets or sets the type of group membership role to check.
        /// </summary>
        [Browsable(true)]
        [Category(FbmlConstants.FBML_OPT_ATTR)]
        [Description("Specifies the group membership role to check.")]
        [DefaultValue(GroupMembershipRole.Member)]
        public GroupMembershipRole Role
        {
            get { return m_role; }
            set
            {
                if (!Enum.IsDefined(typeof(GroupMembershipRole), value))
                    throw new InvalidEnumArgumentException("value", (int)value, typeof(GroupMembershipRole));
                m_role = value;
            }
        }

        /// <inheritdoc />
        protected override void AddAttributesToRender(System.Web.UI.HtmlTextWriter writer)
        {
            base.AddAttributesToRender(writer);

            if (GroupID == 0)
                throw new RequiredFbmlAttributeNotSetException("GroupID", GroupID);
            writer.AddAttribute("gid", GroupID.ToString(CultureInfo.InvariantCulture));
            if (UserID != 0)
                writer.AddAttribute("uid", UserID.ToString(CultureInfo.InvariantCulture));
            if (m_role != GroupMembershipRole.Member)
                writer.AddAttribute("role", m_role.ToString().ToLowerInvariant());
        }

        /// <inheritdoc />
        protected internal override string ElementName
        {
            get { return FbmlConstants.FB_IF_IS_GROUP_MEMBER; }
        }
    }
}
