﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using Terralever.Facebook.UI.Design;
using System.Globalization;
using System.Web.UI;
using System.ComponentModel.Design;
using System.Drawing.Design;

namespace Terralever.Facebook.UI.FbmlControls
{
    /// <summary>
    /// Displays content inside the tag only if the logged-in user (or specified user) is one of the specified users.
    /// </summary>
    /// <remarks>
    /// <para>The Facebook documentation for the rendered control can be found 
    /// <a href="http://wiki.developers.facebook.com/index.php/Fb:if-is-user" target="_blank">here</a>.</para>
    /// </remarks>
    [ToolboxData("<{0}:IfIsUser runat=\"server\"></{0}:IfIsUser>")]
    [DefaultProperty("UserIDs")]
    public class IfIsUser : LogicTestControl
    {
        /// <summary>
        /// Gets or sets the user IDs of the users who are allowed to view the content inside the tag.
        /// </summary>
        [Browsable(true)]
        [Category(FbmlConstants.FBML_REQD_ATTR)]
        [Description("The user IDs of the users who are allowed to see the content.")]
        [TypeConverter(typeof(Int64ArrayTypeConverter))]
        [FbmlRequired(IsRequired = true)]
        [Editor(typeof(ArrayEditor), typeof(UITypeEditor))]
        public long[] UserIDs
        {
            get;
            set;
        }

        /// <inheritdoc />
        protected override void AddAttributesToRender(System.Web.UI.HtmlTextWriter writer)
        {
            base.AddAttributesToRender(writer);

            if (UserIDs == null || UserIDs.Length == 0)
                throw new RequiredFbmlAttributeNotSetException("UserIDs", UserIDs);

            writer.AddAttribute("uid", Utility.FormatArrayToCommaDelimitedString(UserIDs, CultureInfo.InvariantCulture));
        }

        /// <inheritdoc />
        protected internal override string ElementName
        {
            get { return FbmlConstants.FB_IF_IS_USER; }
        }
    }
}
