﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using Terralever.Facebook.UI.Design;
using System.Globalization;
using System.Web.UI;

namespace Terralever.Facebook.UI.FbmlControls
{
    /// <summary>
    /// Displays content inside the tag only if the user is in a given network.
    /// </summary>
    /// <remarks>
    /// <para>The Facebook documentation for the rendered control can be found 
    /// <a href="http://wiki.developers.facebook.com/index.php/Fb:is-in-network" target="_blank">here</a>.</para>
    /// </remarks>
    [ToolboxData("<{0}:IsInNetwork runat=\"server\"></{0}:IsInNetwork>")]
    public class IsInNetwork : LogicTestControl
    {
        /// <inheritdoc />
        protected internal override string ElementName
        {
            get { return FbmlConstants.FB_IS_IN_NETWORK; }
        }

        /// <summary>
        /// Gets or sets the network ID to check.  This property is required.
        /// </summary>
        [FbmlRequired(IsRequired = true)]
        [Browsable(true)]
        [Category(FbmlConstants.FBML_REQD_ATTR)]
        [Description("The network ID to check.")]
        [DefaultValue(0)]
        public long NetworkID
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the user ID to check.  If this property is 0, Facebook will check against the logged-in user.
        /// </summary>
        [Browsable(true)]
        [Category(FbmlConstants.FBML_OPT_ATTR)]
        [Description("The user ID to check.  If this value is 0, it checks against the logged-in user.")]
        [DefaultValue(0)]
        public long UserID
        {
            get;
            set;
        }

        /// <inheritdoc />
        protected override void AddAttributesToRender(System.Web.UI.HtmlTextWriter writer)
        {
            base.AddAttributesToRender(writer);

            if (NetworkID == 0)
                throw new RequiredFbmlAttributeNotSetException("NetworkID", NetworkID);

            writer.AddAttribute("network", NetworkID.ToString(CultureInfo.InvariantCulture));

            if (UserID != 0)
                writer.AddAttribute("uid", UserID.ToString(CultureInfo.InvariantCulture));
        }
    }
}
