﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI;
using Terralever.Facebook.UI.Design;
using System.ComponentModel;

namespace Terralever.Facebook.UI.FbmlControls
{
    /// <summary>
    /// Represents a request button for use on the user's requests page.
    /// </summary>
    [ToolboxData("<{0}:RequestChoice runat=\"server\" />")]
    public class RequestChoice : FbmlButtonBase
    {
        /// <summary>
        /// Gets or sets the URL to which this button links.
        /// </summary>
        /// <remarks>
        /// <para>This value should be the URL to the Facebook Add Application URL or the homepage of the application.</para>
        /// </remarks>
        [FbmlRequired(IsRequired = true)]
        [Browsable(true)]
        [DefaultValue(null)]
        [Category(FbmlConstants.FBML_REQD_ATTR)]
        [Description("Specifies the url to which this button links.")]
        public string Url
        {
            get;
            set;
        }

        /// <inheritdoc />
        protected internal override string ElementName
        {
            get { return FbmlConstants.FB_REQ_CHOICE; }
        }

        /// <inheritdoc />
        protected override void AddAttributesToRender(HtmlTextWriter writer)
        {
            if (string.IsNullOrEmpty(Url))
                throw new RequiredFbmlAttributeNotSetException("Url", Url);

            base.AddAttributesToRender(writer);

            writer.AddAttribute("url", Url);
        }
    }
}
