﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.Web.UI;
using Terralever.Facebook.UI.Design;

namespace Terralever.Facebook.UI.FbmlControls
{
    /// <summary>
    /// Provides common functionality for the <see>Error</see>, <see>Success</see>, and <see>Explanation</see> controls.
    /// </summary>
    [DefaultProperty("Message")]
    [Designer("Terralever.Facebook.UI.Design.FbmlControls.StatusMessageDesigner, " + Constants.DESIGNER_LIB)]
    public abstract class StatusMessageBase : FbmlControl
    {
        /// <summary>
        /// Gets or sets the error message displayed to the user.  This property is required.
        /// </summary>
        [DefaultValue(null)]
        [Category(FbmlConstants.FBML_REQD_ATTR)]
        [Description("Specifies the primary message to display to the user.")]
        [FbmlRequired(IsRequired = true)]
        public string Message
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets descriptive information about the error.
        /// </summary>
        [DefaultValue(null)]
        [Category(FbmlConstants.FBML_OPT_ATTR)]
        [Description("Specifies additional information about the message.")]
        public string Description
        {
            get;
            set;
        }

        /// <inheritdoc />
        protected override void AddAttributesToRender(HtmlTextWriter writer)
        {
            base.AddAttributesToRender(writer);
            if (string.IsNullOrEmpty(Message))
                throw new RequiredFbmlAttributeNotSetException("Message", Message);

            writer.AddAttribute("message", Message, true);
            if (DecorationType != StatusMessageDecorationType.Standard)
            {
                switch (DecorationType)
                {
                    case StatusMessageDecorationType.NoBottomPadding:
                        writer.AddAttribute("decoration", "shorten");
                        break;
                    case StatusMessageDecorationType.NoTopPadding:
                        writer.AddAttribute("decoration", "no_padding");
                        break;
                    default:
                        break;
                }
            }
        }

        /// <inheritdoc />
        protected override void Render(HtmlTextWriter writer)
        {
            if (string.IsNullOrEmpty(Description))
            {
                base.Render(writer);
            }
            else
            {
                AddAttributesToRender(writer);
                writer.RenderBeginTag(ElementName);
                writer.WriteEncodedText(Description);
                writer.RenderEndTag();
            }
        }

        private StatusMessageDecorationType m_deco = StatusMessageDecorationType.Standard;

        /// <summary>
        /// Gets or sets the padding style used for the element.
        /// </summary>
        /// <exception cref="InvalidEnumArgumentException">Thrown if <paramref name="value"/> is not a member of the <see>StatusMessageDecorationType</see>
        ///  enumeration.</exception>
        [DefaultValue(StatusMessageDecorationType.Standard)]
        [Category(FbmlConstants.FBML_OPT_ATTR)]
        [Description("Specifies a padding style for the element.")]
        public StatusMessageDecorationType DecorationType
        {
            get { return m_deco; }
            set
            {
                if (!Enum.IsDefined(typeof(StatusMessageDecorationType), value))
                    throw new InvalidEnumArgumentException("value", (int)value, typeof(StatusMessageDecorationType));
                m_deco = value;
            }
        }

    }
}
