﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using Terralever.Facebook.UI.Design;
using System.Web.UI;
using System.Globalization;

namespace Terralever.Facebook.UI.FbmlControls
{
    /// <summary>
    /// Hides the content enclosed within this control from any user who is blocked by the user referenced with the 
    /// <see>UserID</see> property.
    /// </summary>
    /// <remarks>
    /// <para>The Facebook documentation for this control can be found 
    /// <a href="http://wiki.developers.facebook.com/index.php/Fb:user">here</a>.</para>
    /// </remarks>
    [ToolboxData("<{0}:User runat=\"server\" />")]
    public class User : ContentDisplayControl
    {
        /// <summary>
        /// Gets or sets the user ID to check.  This property is required.
        /// </summary>
        [Browsable(true)]
        [Category(FbmlConstants.FBML_REQD_ATTR)]
        [Description("Specifies the user ID to check.")]
        [DefaultValue(0)]
        [FbmlRequired(IsRequired = true)]
        public long UserID
        {
            get;
            set;
        }

        /// <inheritdoc />
        protected override void AddAttributesToRender(HtmlTextWriter writer)
        {
            if (UserID == 0)
                throw new RequiredFbmlAttributeNotSetException("UserID", UserID);

            writer.AddAttribute("uid", UserID.ToString(CultureInfo.InvariantCulture));

            base.AddAttributesToRender(writer);
        }

        /// <inheritdoc />
        protected internal override string ElementName
        {
            get { return FbmlConstants.FB_USER; }
        }
    }
}
