﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.Serialization;
using System.Globalization;
using System.Security.Permissions;

namespace Terralever.Facebook.UI
{
    /// <summary>
    /// Specifies that a required FBML attribute was not set.
    /// </summary>
    [Serializable]
    public class RequiredFbmlAttributeNotSetException : Exception
    {
        private string m_propName;
        private object m_defValue;

        /// <summary>
        /// Creates a new <see>RequiredFbmlAttributeNotSetException</see>.
        /// </summary>
        public RequiredFbmlAttributeNotSetException() { }
        /// <summary>
        /// Creates a new <see>RequiredFbmlAttributeNotSetException</see>.
        /// </summary>
        /// <param name="message">Informational text about the exception.</param>
        public RequiredFbmlAttributeNotSetException(string message) : base(message) { }
        /// <summary>
        /// Creates a new <see>RequiredFbmlAttributeNotSetException</see>.
        /// </summary>
        /// <param name="message">Informational text about the exception.</param>
        /// <param name="inner">An inner exception wrapped by this exception.</param>
        public RequiredFbmlAttributeNotSetException(string message, Exception inner) : base(message, inner) { }

        /// <summary>
        /// Deserializes a <see>RequiredFbmlAttributeNotSetException</see>.
        /// </summary>
        /// <param name="info">Serialization data.</param>
        /// <param name="context">The serialization streaming context.</param>
        [SecurityPermission(SecurityAction.LinkDemand, SerializationFormatter = true)]
        protected RequiredFbmlAttributeNotSetException(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
            m_propName = info.GetString("PropertyName");
            m_defValue = info.GetValue("InvalidValue", typeof(object));
        }

        /// <summary>
        /// Creates a new <see>RequiredFbmlAttributeNotSetException</see>.
        /// </summary>
        /// <param name="propertyName">The name of the property that was at fault.</param>
        /// <param name="invalidValue">The invalid value, if any, that was the default or set value.</param>
        public RequiredFbmlAttributeNotSetException(string propertyName, object invalidValue) 
            : base(string.Format(CultureInfo.InvariantCulture, "FBML required property \"{0}\" was not set.  The default/invalid value was \"{1}\".", propertyName, invalidValue ?? (object)"{null}"))
        {
            if (propertyName == null) throw new ArgumentNullException("propertyName");

            m_propName = propertyName;
            m_defValue = invalidValue;

        }

        /// <inheritdoc />
        [SecurityPermission(SecurityAction.LinkDemand, SerializationFormatter = true)]
        public override void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            base.GetObjectData(info, context);

            info.AddValue("PropertyName", m_propName);
            info.AddValue("InvalidValue", m_defValue, typeof(object));
        }

        /// <summary>
        /// The property that was not set or set invalidly.
        /// </summary>
        public string PropertyName
        {
            get { return m_propName; }
        }

        /// <summary>
        /// The invalid or default value of the property.
        /// </summary>
        public object InvalidValue
        {
            get { return m_defValue; }
        }
    }
}
